@@ -1,3 +1,8 @@
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include "gd.h"
 
@@ -39,7 +44,7 @@ main (int argc, char **argv)
 #ifdef HAVE_LIBPNG
   gdImagePng (im, out);
 #else
-  fprintf(stderr, "No PNG library support.\n");
+  fprintf (stderr, "No PNG library support.\n");
 #endif
   fclose (out);
   gdImageDestroy (im);
